package me.km.datatools;

import me.km.module.Module;
import me.km.module.ModuleCommand;
import me.km.utils.Utils;
import me.km.utils.NBTUtils;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import me.km.permissions.Permissions;
import me.km.utils.ItemStackUtils;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.ItemArmor;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.TextComponentString;

public class CommandDataTools extends ModuleCommand
{
    private final ArrayList<String> first;
    
    public CommandDataTools(Module m) 
    {
        super("datatools", m);
        super.setDescription("special commands for changing date");
        super.setUsage("/datatools for help");
        super.setPermission(Permissions.DATATOOLS);  
        super.addAlias("dt");
        
        first = new ArrayList<>(Arrays.asList(new String[]
                {"print-nbt", "flag", "attribute", "hide-entity", "show-entity", 
            "name-entity", "name-item", "lore-item", "silent-entity", "magic"})); 
    }

    @Override
    public List<String> getTabCompletions(MinecraftServer server, ICommandSender sender, String[] args, BlockPos targetPos) 
    {
        List<String> names = new ArrayList<>();
        switch (args.length) 
        {
            case 1:
                first.stream().filter(e -> e.toLowerCase().startsWith(args[0].toLowerCase())).forEach(e ->
                {
                    names.add(e);
                }); 
                break;
            case 2:
                if(args[0].equals("flag"))
                {
                    Arrays.stream(ItemStackUtils.ItemFlag.values()).filter(e -> e.toString().toLowerCase().startsWith(args[1].toLowerCase())).forEach(e ->
                    {
                        names.add(e.toString());
                    });
                }
                else if(args[0].equals("attribute"))
                {
                    Arrays.stream(ItemStackUtils.Attribute.values()).filter(e -> e.toString().toLowerCase().startsWith(args[1].toLowerCase())).forEach(e ->
                    {
                        names.add(e.toString());
                    });
                }   
                break;
            case 3:
                if(args[0].equals("attribute"))
                {
                    Arrays.stream(EntityEquipmentSlot.values()).filter(e -> e.toString().toLowerCase().startsWith(args[2].toLowerCase())).forEach(e ->
                    {
                        names.add(e.toString());
                    });
                }   
                break;
            case 5:
                if(args[0].equals("attribute"))
                {
                    Arrays.stream(ItemStackUtils.Operation.values()).filter(e -> e.toString().toLowerCase().startsWith(args[4].toLowerCase())).forEach(e ->
                    {
                        names.add(e.toString());
                    });
                }   
                break;    
        }
        return names;  
    }

    @Override
    public boolean execute(ICommandSender cs, String[] arg) 
    {
        if(!(cs instanceof EntityPlayerMP))
        {
            //this.getModule().send(cs, GlobalText.onlyPlayer());
            return true;
        }
        EntityPlayerMP p = (EntityPlayerMP) cs;
        Module m = this.getModule();
        if(arg.length >= 1)
        {         
            switch (arg[0]) 
            {
                case "print-nbt":
                {
                    Entity ent = Utils.getTargetedEntity(p, 3, Entity.class);
                    if(ent == null)
                    {
                        m.send(cs, "You must look at an entity.");
                        return true;
                    }       
                    //this.getModule().send(cs, GlobalText.Spacer());
                    this.getModule().send(cs, "NBT: " + ent.getName());
                    String nbt = NBTUtils.toString(ent);
                    cs.sendMessage(new TextComponentString(NBTUtils.highlightNbtSyntax(nbt)));   
                    NBTUtils.sendCopyableText(p, nbt);
                    return true;
                }
                case "flag":
                {
                    if(arg.length < 2)
                    {
                        break;
                    }
                    ItemStack stack = p.getHeldItemMainhand();
                    if(stack == ItemStack.EMPTY)
                    {
                        m.send(cs, "You must hold an item in your hand.");
                        return true;
                    }
                    try
                    {
                        ItemStackUtils.ItemFlag flag = ItemStackUtils.ItemFlag.valueOf(arg[1].toUpperCase());
                        ItemStackUtils.addItemFlag(stack, flag);
                        m.send(cs, "The flag was added.");
                    }
                    catch(IllegalArgumentException ex)
                    {
                        m.send(cs, "The given flag does not exist.");
                    }                           
                    return true;
                }
                case "attribute":
                {
                    ItemStack stack = p.getHeldItemMainhand();
                    if(stack == ItemStack.EMPTY)
                    {
                        m.send(cs, "You must hold an item in your hand.");
                        return true;
                    }                            
                    if(arg.length < 5)
                    {
                        break;
                    }
                    try
                    {
                        ItemStackUtils.addAttribute(stack, ItemStackUtils.Attribute.valueOf(arg[1]), 
                                EntityEquipmentSlot.valueOf(arg[2]), Double.parseDouble(arg[3]), ItemStackUtils.Operation.valueOf(arg[4]));
                        m.send(cs, "The attribute was added.");
                    } 
                    catch(NumberFormatException ex)
                    {
                        m.send(cs, "A number is invalid.");
                    }
                    catch(IllegalArgumentException ex)
                    {
                        m.send(cs, "The enum does not exist.");
                    }
                    return true;
                }
                case "hide-entity":
                {
                    Entity ent = Utils.getTargetedEntity(p, 3, Entity.class);
                    if(ent == null)
                    {
                        m.send(cs, "You must look at an entity.");
                        return true;
                    }                        
                    ent.setInvisible(true);
                    m.send(cs, "The entity is now invisible.");
                    return true;    
                }
                case "show-entity":
                    Collection<Entity> ents = Utils.getEntities(p.getEntityWorld(), p.posX, p.posY, p.posZ, 3, Entity.class);
                    if(ents.isEmpty())
                    {
                        m.send(cs, "There must be at least one entity in your surroundings.");
                        return true;
                    } 
                    ents.stream().forEach(ent -> ent.setInvisible(false));
                    m.send(cs, "All entities near you are visible again.");
                    return true;  
                case "silent-entity":
                {
                    Entity ent = Utils.getTargetedEntity(p, 3, Entity.class);
                    if(ent == null)
                    {
                        m.send(cs, "You must look at an entity.");
                        return true;
                    }                                 
                    ent.setSilent(true);
                    m.send(cs, "The entity is now silent.");
                    return true;    
                }
                case "name-entity":
                {
                    if(arg.length < 2)
                    {
                        break;
                    }
                    Entity ent = Utils.getTargetedEntity(p, 3, Entity.class);
                    if(ent == null)
                    {
                        m.send(cs, "You must look at an entity.");
                        return true;
                    }                                
                    String s = Utils.colorMessage(Utils.connectSpaces(arg, 1), p);
                    ent.setCustomNameTag(s);
                    m.send(cs, "The entity is now called '" + s + "§r'.");
                    return true; 
                }
                case "name-item":
                {
                    if(arg.length < 2)
                    { 
                        break;
                    }
                    ItemStack hand = p.getHeldItemMainhand();
                    if(hand == ItemStack.EMPTY)
                    {
                        m.send(cs, "You must hold an item in your hand.");
                        return true;
                    }                          
                    String s = Utils.colorMessage(Utils.connectSpaces(arg, 1), p);
                    hand.setStackDisplayName(s);
                    m.send(cs, "The item is now called '" + s + "§r'.");
                    return true;
                }
                case "lore-item":
                {
                    if(arg.length < 3)
                    {
                        break;
                    }
                    ItemStack hand = p.getHeldItemMainhand();
                    if(hand == ItemStack.EMPTY)
                    {
                        m.send(cs, "You must hold an item in your hand.");
                        return true;
                    }                          
                    int line;
                    try
                    {
                        line = Integer.parseInt(arg[1]);
                        if(line < 0)
                        {
                            throw new NumberFormatException();
                        }
                    }
                    catch(NumberFormatException ex)
                    {
                        m.send(cs, "An invalid number was given.");
                        return true;
                    }            
                    ItemStackUtils.addLore(hand, Utils.colorMessage(Utils.connectSpaces(arg, 2), p), line);
                    m.send(cs, "The new lore was added.");
                    return true;
                }
                case "magic":
                {
                    if(arg.length < 2)
                    {
                        break;
                    }
                    ItemStack hand = p.getHeldItemMainhand();
                    if(hand == ItemStack.EMPTY)
                    {
                        m.send(cs, "You must hold an item in your hand.");
                        return true;
                    }                          
                    int value;
                    try
                    {
                        value = Integer.parseInt(arg[1]);
                        if(value < 0)
                        {
                            throw new NumberFormatException();
                        }
                    }
                    catch(NumberFormatException ex)
                    {
                        m.send(cs, "An invalid number was given.");
                        return true;
                    }         
                    if(hand.getItem() instanceof ItemArmor)
                    {
                        NBTTagCompound com = hand.getTagCompound();
                        if(com == null)
                        {
                            com = new NBTTagCompound();
                            hand.setTagCompound(com);
                        }
                        com.setInteger("magic", value);
                        m.send(cs, "The magic resistance was set to " + value + ".");
                    }
                    else
                    {
                        m.send(cs, "Only armor can have magic resistance.");
                    }
                    return true;
                }
            }
        }
                    
        m.send(cs, "/datatools ...");
        m.sendHelpListElement(cs, "print-nbt", "prints NBT of an entity");
        m.sendHelpListElement(cs, "flag <flag>", "adds a flag to an itemstack");
        m.sendHelpListElement(cs, "attribute <a> <slot> <d> <op>", "adds an attribute to an itemstack");
        m.sendHelpListElement(cs, "hide-entity", "makes an entity invisible");
        m.sendHelpListElement(cs, "show-entity", "makes near entities visible");
        m.sendHelpListElement(cs, "silent-entity", "silence an entity");
        m.sendHelpListElement(cs, "name-entity <name>", "sets the name of an entity");
        m.sendHelpListElement(cs, "name-item <name>", "sets the name of an item");
        m.sendHelpListElement(cs, "lore-item <line> <lore>", "sets the lore of an item");
        m.sendHelpListElement(cs, "magic <def>", "sets magic resistance of armor");
        return true;  
    }
}
