package me.km.events;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import me.hammerle.snuviscript.code.Script;
import me.km.DamageUtils;
import me.km.KajetansMod;
import me.km.utils.ReflectionUtils;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.entity.projectile.EntityThrowable;
import net.minecraft.util.CombatRules;
import net.minecraft.util.DamageSource;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.EntityJoinWorldEvent;
import net.minecraftforge.event.entity.living.LivingHurtEvent;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;

public class CustomEventCaller 
{
    private static final ArrayList<EntityArrow> ARROWS = new ArrayList<>();
    
    private static final HashMap<Integer, PlayerMoveData> MOVE_DATA = new HashMap<>();
    
    private static final LinkedList<Integer> REMOVE_QUEUE = new LinkedList<>();
    private static final LinkedList<PlayerMoveData> ADD_QUEUE = new LinkedList<>();
    
    public static int registerMoveData(PlayerMoveData data)
    {
        ADD_QUEUE.add(data);
        return data.getId();
    }
    
    public static void removeScriptData(Script sc)
    {
        MOVE_DATA.entrySet().forEach(e -> 
        {
            if(e.getValue().isSameScript(sc))
            {
                REMOVE_QUEUE.add(e.getValue().getId());
            }
        });
    }
    
    public static void removeMoveData(int id)
    {
        REMOVE_QUEUE.add(id);
    }
    
    @SubscribeEvent
    public void onPlayerMove(TickEvent.ServerTickEvent e)
    {      
        if(e.phase == TickEvent.Phase.END)
        {
            if(!ADD_QUEUE.isEmpty())
            {
                ADD_QUEUE.forEach(data -> MOVE_DATA.put(data.getId(), data));
                ADD_QUEUE.clear();
            }
            MOVE_DATA.values().removeIf(data -> 
            {
                if(data.tickLiving())
                {
                    return true;
                }
                if(data.tick())
                {
                    return false;
                }
                return KajetansMod.server.getPlayerList().getPlayers().stream().anyMatch(p -> data.check(p));
            });
            if(!REMOVE_QUEUE.isEmpty())
            {
                REMOVE_QUEUE.forEach(i -> MOVE_DATA.remove(i));
                REMOVE_QUEUE.clear();
            }
            
            ARROWS.removeIf(arrow -> 
            {
                if(ReflectionUtils.getArrowTimeInGround(arrow) >= 1)
                {
                    MinecraftForge.EVENT_BUS.post(new ArrowHitGroundEvent(arrow));
                    return true;
                }
                return false;
            });
        }
    } 
    
    @SubscribeEvent
    public void onArrowLaunch(EntityJoinWorldEvent e)
    {       
        Entity ent = e.getEntity();
        if(ent instanceof EntityArrow)
        {
            EntityArrow arrow = (EntityArrow) ent;
            if(arrow.shootingEntity instanceof EntityPlayer)
            {
                ARROWS.add(arrow);
                MinecraftForge.EVENT_BUS.post(new ArrowLaunchEvent((EntityPlayer) arrow.shootingEntity, arrow, e.getWorld()));
            }
        }
        else if(ent instanceof EntityThrowable)
        {
            EntityThrowable thrown = (EntityThrowable) ent;
            if(thrown.getThrower() instanceof EntityPlayer)
            {
                MinecraftForge.EVENT_BUS.post(new ThrowableLaunchEvent((EntityPlayer) thrown.getThrower(), thrown, e.getWorld()));
            }
        }
    }
    
    @SubscribeEvent
    public void onPlayerHurt(LivingHurtEvent e)
    {      
        if(e.getEntityLiving() instanceof EntityPlayerMP && !(e instanceof PlayerHurtEvent))
        {
            PlayerHurtEvent hurt = new PlayerHurtEvent((EntityPlayerMP) e.getEntityLiving(), e.getSource(), e.getAmount());
            MinecraftForge.EVENT_BUS.post(hurt);
            e.setAmount(hurt.getAmount());
            e.setCanceled(hurt.isCanceled());
        }
    } 
    
    private float getRealDamage(float damageAmount, DamageSource ds, EntityLivingBase liv)
    {
        if(ds != DamageSource.LAVA && ds != DamageSource.IN_WALL && ds != DamageSource.CRAMMING && ds != DamageSource.DROWN)
        {
            if(ds.isMagicDamage() && !ds.isDamageAbsolute()) // Magic Damage
            {
                ReflectionUtils.damageArmor(liv, damageAmount);
                int armor = DamageUtils.getMagicDefense(liv);
                damageAmount = CombatRules.getDamageAfterAbsorb(damageAmount, armor, 0);
            }
            else if(!ds.isUnblockable())
            {
                ReflectionUtils.damageArmor(liv, damageAmount);
                int armor = liv.getTotalArmorValue();
                damageAmount = CombatRules.getDamageAfterAbsorb(damageAmount, armor, (float) liv.getEntityAttribute(SharedMonsterAttributes.ARMOR_TOUGHNESS).getAttributeValue());
            }           
            damageAmount = ReflectionUtils.applyPotionDamageCalculations(liv, ds, damageAmount);
        }
        else
        {
            damageAmount = liv.getMaxHealth() / 10;
        }
        return damageAmount;
    }
    
    @SubscribeEvent(priority = EventPriority.LOWEST)
    public void newDamageSystem(LivingHurtEvent e)
    {         
        // ---------------------------------------------------------------------
        // injection of new damage system
        // ---------------------------------------------------------------------
        if(!e.isCanceled())
        {
            e.setCanceled(true);

            DamageSource ds = e.getSource();
            EntityLivingBase liv = e.getEntityLiving();

            float damageAmount = getRealDamage(e.getAmount(), ds, liv);
            float f = damageAmount;
            damageAmount = Math.max(damageAmount - liv.getAbsorptionAmount(), 0.0F);
            liv.setAbsorptionAmount(liv.getAbsorptionAmount() - (f - damageAmount));

            if (damageAmount != 0.0F)
            {
                float f1 = liv.getHealth();
                liv.setHealth(f1 - damageAmount);
                liv.getCombatTracker().trackDamage(ds, f1, damageAmount);
                liv.setAbsorptionAmount(liv.getAbsorptionAmount() - damageAmount);
            }
        }
    }
}
