package me.km.items.weapons;

import javax.annotation.Nullable;
import me.km.entities.EntityItemProjectile;
import me.km.items.ItemBase;
import me.km.items.ModItems;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.EnumAction;
import net.minecraft.item.IItemPropertyGetter;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.stats.StatList;
import net.minecraft.util.ActionResult;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumHand;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class ItemSling extends ItemBase
{
    public ItemSling(String name, String local) 
    {
        super(name, local);
        super.maxStackSize = 1;
        super.setMaxDamage(192);
        super.setCreativeTab(CreativeTabs.COMBAT);
        this.addPropertyOverride(new ResourceLocation("pull"), new IItemPropertyGetter()
        {
            @SideOnly(Side.CLIENT)
            @Override
            public float apply(ItemStack stack, @Nullable World w, @Nullable EntityLivingBase ent)
            {
                if (ent == null)
                {
                    return 0f;
                }
                else
                {
                    return ent.getActiveItemStack().getItem() != ModItems.sling ? 0f : (stack.getMaxItemUseDuration() - ent.getItemInUseCount()) / 20.0F;
                }
            }
        });
        this.addPropertyOverride(new ResourceLocation("pulling"), new IItemPropertyGetter()
        {
            @SideOnly(Side.CLIENT)
            @Override
            public float apply(ItemStack stack, @Nullable World w, @Nullable EntityLivingBase ent)
            {
                return ent != null && ent.isHandActive() && ent.getActiveItemStack() == stack ? 1f : 0f;
            }
        });
    }
    
    private ItemStack findAmmo(EntityPlayer p)
    {
        if(this.isAmmo(p.getHeldItem(EnumHand.OFF_HAND)))
        {
            return p.getHeldItem(EnumHand.OFF_HAND);
        }
        else if(this.isAmmo(p.getHeldItem(EnumHand.MAIN_HAND)))
        {
            return p.getHeldItem(EnumHand.MAIN_HAND);
        }
        else
        {
            ItemStack stack;
            for(int i = 0; i < p.inventory.getSizeInventory(); i++)
            {
                stack = p.inventory.getStackInSlot(i);
                if(this.isAmmo(stack))
                {
                    return stack;
                }
            }
            return ItemStack.EMPTY;
        }
    }
    
    private boolean isAmmo(ItemStack stack)
    {
        return stack.getItem() == Item.getItemFromBlock(Blocks.COBBLESTONE);
    }

    @Override
    public void onPlayerStoppedUsing(ItemStack stack, World w, EntityLivingBase liv, int timeLeft)
    {
        if(liv instanceof EntityPlayer)
        {
            int i = this.getMaxItemUseDuration(stack) - timeLeft;
            if(i < 0)
            {
                return;
            }            
            EntityPlayer p = (EntityPlayer) liv;
            ItemStack ammo = this.findAmmo(p);
            if(!ammo.isEmpty() || p.capabilities.isCreativeMode)
            {
                if(ammo.isEmpty())
                {
                    ammo = new ItemStack(Blocks.COBBLESTONE);
                }
                
                float f = getArrowVelocity(i);
                if(f >= 0.1)
                {
                    if(!w.isRemote)
                    {
                        ItemStack ammoCopy = ammo.copy();
                        ammoCopy.setCount(1);
                        EntityItemProjectile proj = new EntityItemProjectile(p, ammoCopy, 2.0f);
                        proj.shoot(p, p.rotationPitch, p.rotationYaw, f, 1.0F);                     
                        stack.damageItem(1, p);
                        w.spawnEntity(proj);
                    }
                    
                    w.playSound(null, p.posX, p.posY, p.posZ, SoundEvents.ENTITY_ARROW_SHOOT, SoundCategory.PLAYERS, 1f, 1f / (itemRand.nextFloat() * 0.4f + 1.2f) + f * 0.5f);
                    
                    if(!p.capabilities.isCreativeMode)
                    {
                        ammo.shrink(1);
                        if(ammo.isEmpty())
                        {
                            p.inventory.deleteStack(ammo);
                        }
                    }
                    p.addStat(StatList.getObjectUseStats(this));
                }
            }
        }
    }

    public static float getArrowVelocity(int charge)
    {
        float f = charge / 20.0F;
        f = (f * f + f * 2.0F) / 3.0F;
        if(f > 1.0F)
        {
            f = 1.0F;
        }
        return f;
    }

    @Override
    public int getMaxItemUseDuration(ItemStack stack)
    {
        return 72000;
    }

    @Override
    public ActionResult<ItemStack> onItemRightClick(World w, EntityPlayer p, EnumHand handIn)
    {
        ItemStack ammo = p.getHeldItem(handIn);
        if(!p.capabilities.isCreativeMode && this.findAmmo(p).isEmpty())
        {
            return new ActionResult(EnumActionResult.FAIL, ammo);
        }
        p.setActiveHand(handIn);
        return new ActionResult<>(EnumActionResult.SUCCESS, ammo);
    }

    @Override
    public int getItemEnchantability()
    {
        return 1;
    }
    
    @Override
    public EnumAction getItemUseAction(ItemStack stack)
    {
        return EnumAction.BOW;
    }
}
