package me.km.module;

import me.km.utils.Utils;
import java.lang.reflect.InvocationTargetException;
import me.km.KajetansMod;
import me.km.PackageHelper;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.util.text.TextFormatting;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.common.event.FMLServerStartingEvent;
import net.minecraftforge.fml.common.eventhandler.EventBus;

public class Module
{
    private final ITextComponent prefixes;
    private final TextFormatting colors;
    
    public Module(String prefix, TextFormatting color)
    {
        TextComponentString text = new TextComponentString(prefix);
        text.getStyle().setColor(color);
        prefixes = new TextComponentString("[").appendSibling(text).appendText("] ");
        colors = color;
        
        sendToConsole("Module " + color + prefix + TextFormatting.RESET + " was loaded.");
    }
    
    //--------------------------------------------------------------------------
    // messages
    //--------------------------------------------------------------------------

    public final void send(ICommandSender cs, String msg)
    {
        if(cs == null)
        {
            return;
        }
        cs.sendMessage(prefixes.createCopy().appendText(msg));
    }

    public final void sendToPlayers(World w, double x, double y, double z, double radius, String msg)
    {
        ITextComponent s = prefixes.createCopy().appendText(msg);
        Utils.getEntities(w, x, y, z, radius, EntityPlayer.class).forEach(p -> ((EntityPlayer) p).sendMessage(s));
    }
    
    public final void sendBroadcast(String msg)
    {
        KajetansMod.server.getPlayerList().sendMessage(prefixes.createCopy().appendText(msg));
    }
    
    public final void sendListElement(ICommandSender cs, String msg)
    {
        if(cs == null)
        {
            return;
        }
        TextComponentString text = new TextComponentString("");
        TextComponentString text2 = new TextComponentString(" - ");
        text2.getStyle().setColor(colors);
        text.appendSibling(text2);
        cs.sendMessage(text.appendText(msg));
    } 
    
    public final void sendHelpListElement(ICommandSender cs, String msg, String msg2)
    {
        if(cs == null)
        {
            return;
        }
        TextComponentString text = new TextComponentString("");
        TextComponentString text2 = new TextComponentString(" - " + msg + " ");
        text2.getStyle().setColor(colors);
        text.appendSibling(text2);
        cs.sendMessage(text.appendText(msg2));
    }
    
    public final void sendWarning(ICommandSender cs, String msg)
    {
        if(cs == null)
        {
            return;
        }
        TextComponentString text = new TextComponentString(msg);
        text.getStyle().setColor(TextFormatting.RED);
        cs.sendMessage(prefixes.createCopy().appendSibling(text));
    }
   
    public final void sendToConsole(String msg)
    {
        send(KajetansMod.server, msg);
    }
    
    public final void sendListElementToConsole(String msg)
    {
        sendListElement(KajetansMod.server, msg);
    }
    
    public final void sendHelpListElementToConsole(String msg, String msg2)
    {
        sendHelpListElement(KajetansMod.server, msg, msg2);
    }
    
    public final void sendWarningToConsole(String msg)
    {
        sendWarning(KajetansMod.server, msg);
    }
    
    @SuppressWarnings("unchecked")
    public void registerCommands(FMLServerStartingEvent e, String packageName)
    {
        PackageHelper.getClasses(packageName).stream().filter(c -> ModuleCommand.class.isAssignableFrom(c)).forEach((Class c) -> 
        {
            try 
            {
                ModuleCommand command = (ModuleCommand) c.getDeclaredConstructor(Module.class).newInstance(this);
                e.registerServerCommand(command);
            }
            catch(ClassCastException | NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException ex)
            { 
                this.sendToConsole("Command '" + c + "' has thrown an error.");
            }
        });
        this.sendToConsole("All commands were loaded.");
    }
 
    @SuppressWarnings("unchecked")
    public void registerEvents(String packageName)
    {
        EventBus bus = MinecraftForge.EVENT_BUS;
        PackageHelper.getClasses(packageName).stream().filter(c -> ModuleListener.class.isAssignableFrom(c)).forEach((Class c) -> 
        {
            try 
            {   
                bus.register((ModuleListener) c.getDeclaredConstructor(Module.class).newInstance(this));
            }
            catch(NoSuchMethodException | ClassCastException | SecurityException | InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException ex)
            {
                this.sendToConsole("Error in event class " + c + " " + ex.toString());
            }
        });
        this.sendToConsole("All events were loaded.");
    }
    
    public final void registerEvent(Object o)
    {
        MinecraftForge.EVENT_BUS.register(o);
    }
}
