package me.km.snuviscript;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import me.hammerle.snuviscript.code.Compiler;
import me.hammerle.snuviscript.code.Script;
import me.kcm.events.PlayerPreRespawnEvent;
import me.kcm.events.PlayerTabListNameEvent;
import me.km.KajetansMod;
import me.km.utils.Utils;
import me.km.world.ModDimensions;
import me.km.entities.EntityHuman;
import me.km.entities.EntityItemProjectile;
import me.km.events.PlayerHurtEvent;
import me.km.utils.Location;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.passive.EntitySheep;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Items;
import net.minecraft.inventory.ClickType;
import net.minecraft.item.ItemStack;
import net.minecraft.server.management.PlayerList;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.text.TextComponentString;
import net.minecraftforge.event.CommandEvent;
import net.minecraftforge.event.ServerChatEvent;
import net.minecraftforge.event.entity.EntityMountEvent;
import net.minecraftforge.event.entity.ThrowableImpactEvent;
import net.minecraftforge.event.entity.item.ItemTossEvent;
import net.minecraftforge.event.entity.living.*;
import net.minecraftforge.event.entity.player.EntityItemPickupEvent;
import net.minecraftforge.event.entity.player.FillBucketEvent;
import net.minecraftforge.event.entity.player.ItemFishedEvent;
import net.minecraftforge.event.entity.player.PlayerDropsEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.event.world.BlockEvent;
import net.minecraftforge.event.world.ExplosionEvent;
import net.minecraftforge.fml.common.eventhandler.Event;
import net.minecraftforge.fml.common.eventhandler.EventPriority;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.PlayerEvent;

public class ScriptEvents
{               
    // -------------------------------------------------------------------------
    // basics
    // -------------------------------------------------------------------------
    
    private static void handleEvent(EntityPlayer p, String event, Consumer<Script> before, Consumer<Script> after)
    {      
        KajetansMod.scripts.getSnuviParser().callEvent(event, (sc) -> 
        {
            if(p != null)
            {
                ScriptVars.setPlayerVars(sc, p); 
            }
            before.accept(sc);
        }, after);
        
        if(p != null)
        {
            Script data = KajetansMod.scripts.getScript(p);
            if(data != null)
            {
                KajetansMod.scripts.getSnuviParser().callEvent(event, data, (sc) -> 
                {
                    ScriptVars.setPlayerVars(sc, p); 
                    before.accept(sc);
                }, after);
            }
        }      
    }

    private static void handleEvent(EntityPlayer p, String event, Consumer<Script> before)
    {      
        handleEvent(p, event, before, null);
    }
    
    private void simpleCancel(Script sc, Event e, String name)
    {
        try
        {
            e.setCanceled(sc.getVar("cancel").getBoolean(sc)); 
        }
        catch(Exception ex)
        {
            KajetansMod.scripts.logger.print("invalid var in '" + name + "' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
        }
    }
         
    // -------------------------------------------------------------------------
    // events
    // -------------------------------------------------------------------------

    public static void onPlayerMove(EntityPlayer p, int id)
    {      
        handleEvent(p, "player_move", sc -> 
        {
            sc.setVar("id", (double) id);
        });
    }
    
    public static boolean onInventoryClick(Script script, SnuviInventory inv, int slot, ClickType click, EntityPlayer p)
    {
        KajetansMod.scripts.getSnuviParser().callEvent("inv_click", script, sc -> 
        {
            ScriptVars.setPlayerVars(sc, p); 
            sc.setVar("inv_id", (double) inv.getId());
            sc.setVar("inv_name", inv.getName());
            sc.setVar("inv_slot", (double) slot);
            ScriptVars.setItemVars(sc, inv.getStackInSlot(slot));
            sc.setVar("cancel", false); 
        }, null);
        return script.getVar("cancel").getBoolean(script);
    }

    public static void onInventoryClose(Script script, SnuviInventory inv, EntityPlayer p)
    {
        KajetansMod.scripts.getSnuviParser().callEvent("inv_close", script, sc -> 
        {
            ScriptVars.setPlayerVars(sc, p); 
            sc.setVar("inv_id", (double) inv.getId());
            sc.setVar("inv_name", inv.getName());
        }, null);
    }
    
    public static void onHumanHurt(EntityPlayer p, EntityHuman h)
    {
        handleEvent(p, "human_hurt", sc -> ScriptVars.setEntityVars(sc, h));
    }
    
    @SubscribeEvent
    public void onPlayerPostRespawn(PlayerEvent.PlayerRespawnEvent e)
    {
        handleEvent(e.player, "player_post_respawn", sc -> {});
    }
    
    @SubscribeEvent
    public void onPlayerPreRespawn(PlayerPreRespawnEvent e)
    {
        handleEvent(e.getEntityPlayer(), "player_pre_respawn", sc -> {});
    }
    
    @SubscribeEvent
    public void onPlayerDamage(PlayerHurtEvent e)
    {        
        EntityPlayer p = e.getEntityPlayer();  
        handleEvent(p, "player_hurt", (sc) -> 
        {
            sc.setVar("player_killed", p.getHealth() <= e.getAmount());   
            sc.setVar("player_damage", (double) e.getAmount());   
            sc.setVar("player_damage_cause", e.getSource().getDamageType());
            EntityPlayer ent = Utils.getDamager(e.getSource());
            if(ent != null)
            {
                sc.setVar("player_involved", true);
                ScriptVars.setSecPlayer(sc, ent);
            }
            else
            {
                sc.setVar("player_involved", false);
            }
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            try
            {
                e.setAmount(sc.getVar("player_damage").getFloat(sc));
                e.setCanceled(sc.getVar("cancel").getBoolean(sc)); 
            }
            catch(Exception ex)
            {
                KajetansMod.scripts.logger.print("invalid var in 'player_hurt' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
            }
        });
    } 
    
    @SubscribeEvent
    public void onPlayerAttack(LivingAttackEvent e)
    {        
        EntityLivingBase liv = e.getEntityLiving();
        if(liv instanceof EntityPlayer)
        {
            EntityPlayer p = (EntityPlayer) liv;
            handleEvent(p, "player_is_attacked", (sc) -> 
            {
                sc.setVar("player_killed", p.getHealth() <= e.getAmount());   
                sc.setVar("player_damage_cause", e.getSource().getDamageType());
                EntityPlayer ent = Utils.getDamager(e.getSource());
                if(ent != null)
                {
                    sc.setVar("player_involved", true);
                    ScriptVars.setSecPlayer(sc, ent);
                }
                else
                {
                    sc.setVar("player_involved", false);
                }
                sc.setVar("cancel", e.isCanceled()); 
            }, (sc) -> 
            {
                simpleCancel(sc, e, "player_is_attacked");
            });
        }
        else
        {
            EntityPlayer p = Utils.getDamager(e.getSource());
            if(p != null)
            {
                handleEvent(p, "player_attacks", (sc) -> 
                {
                    ScriptVars.setEntityVars(sc, liv);
                    sc.setVar("cancel", e.isCanceled()); 
                }, (sc) -> 
                {
                    simpleCancel(sc, e, "player_attacks");
                });
            }
        }
    } 
    
    @SubscribeEvent
    public void onPlayerDamage(LivingHealEvent e)
    {        
        if(e.getEntityLiving() instanceof EntityPlayer)
        {
            handleEvent((EntityPlayer) e.getEntityLiving(), "player_heal", (sc) -> 
            {
                sc.setVar("heal", (double) e.getAmount());
                sc.setVar("cancel", e.isCanceled()); 
            }, (sc) -> 
            {
                try
                {
                    e.setAmount(sc.getVar("heal").getFloat(sc));
                    e.setCanceled(sc.getVar("cancel").getBoolean(sc)); 
                }
                catch(Exception ex)
                {
                    KajetansMod.scripts.logger.print("invalid var in 'player_heal' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
                }
            });
        }
    } 
    
    @SubscribeEvent
    public void onLivingDeath(LivingDeathEvent e)
    {        
        if(!(e.getEntityLiving() instanceof EntityPlayer))
        {
            handleEvent(null, "entity_death", (sc) -> 
            {
                EntityLivingBase ent = e.getEntityLiving();
                ScriptVars.setEntityVars(sc, ent);
                EntityPlayer p = Utils.getDamager(e.getSource());
                if(p != null)
                {
                    sc.setVar("player_involved", true);
                    ScriptVars.setPlayerVars(sc, p);
                }  
                else
                {
                    sc.setVar("player_involved", false);
                }
                sc.setVar("cancel", e.isCanceled()); 
            }, (sc) -> 
            {
                simpleCancel(sc, e, "entity_death");
            });
            return;
        }
        EntityPlayer p = (EntityPlayer) e.getEntity();    
        handleEvent(p, "player_death", (sc) -> 
        {
            sc.setVar("clear", false);
            EntityPlayer ent = Utils.getDamager(e.getSource());
            if(ent != null)
            {
                ScriptVars.setSecPlayer(sc, ent);
            }
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            try
            {
                if(sc.getVar("clear").getBoolean(sc))
                {
                    p.inventory.clear();
                }
                e.setCanceled(sc.getVar("cancel").getBoolean(sc)); 
            }
            catch(Exception ex)
            {
                KajetansMod.scripts.logger.print("invalid var in 'player_death' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
            }
        });
    } 
       
    @SubscribeEvent
    public void onEntityDamage(LivingHurtEvent e)
    {        
        EntityPlayer p = Utils.getDamager(e.getSource());
        if(p == null)
        {
            return;
        }
        handleEvent(p, "entity_hurt", (sc) -> 
        {
            sc.setVar("entity_killed", e.getEntityLiving().getHealth() <= e.getAmount());        
            ScriptVars.setEntityVars(sc, e.getEntity()); 
            sc.setVar("entity_damage", (double) e.getAmount());   
            sc.setVar("entity_damage_cause", e.getSource().getDamageType());
            sc.setVar("cancel", e.isCanceled());   
        }, (sc) -> 
        {
            try
            {
                e.setAmount(sc.getVar("entity_damage").getFloat(sc));
                e.setCanceled(sc.getVar("cancel").getBoolean(sc)); 
            }
            catch(Exception ex)
            {
                KajetansMod.scripts.logger.print("invalid var in 'entity_hurt' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
            }
        });
    }
    
    @SubscribeEvent
    public void onEntityDrop(LivingDropsEvent e)
    {             
        try
        {
            handleEvent(null, "entity_drop", (sc) -> 
            {
                sc.setVar("drops", e.getDrops());
                ScriptVars.setEntityVars(sc, e.getEntityLiving());
                EntityPlayer p = Utils.getDamager(e.getSource());
                if(p != null)
                {
                    sc.setVar("player_involved", true);
                    ScriptVars.setPlayerVars(sc, p);
                }  
                else
                {
                    sc.setVar("player_involved", false);
                }
                sc.setVar("cancel", e.isCanceled()); 
            }, (sc) -> 
            {
                simpleCancel(sc, e, "entity_drop");
            });
        }
        catch(NullPointerException ex)
        {
            KajetansMod.scripts.sendWarningToConsole(ex.toString() + "  " + ex.getMessage());
        }
    } 
            
    @SubscribeEvent
    public void onPlayerDrop(PlayerDropsEvent e)
    {             
        handleEvent(e.getEntityPlayer(), "player_drop", (sc) -> 
        {
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "player_drop");
        });
    } 
    
    @SubscribeEvent
    public void onProjectileHit(ThrowableImpactEvent e)
    {        
        EntityPlayer p = Utils.getPlayerFromProjectile(e.getEntityThrowable());
        if(p != null)
        {
            handleEvent(p, "throw_hit", (sc) -> 
            {
                if(e.getRayTraceResult().entityHit != null)
                {
                    sc.setVar("is_entity_hit", true);
                    sc.setVar("entity_hit", e.getRayTraceResult().entityHit);
                }
                else
                {
                    sc.setVar("is_entity_hit", false);
                }
                ScriptVars.setEntityVars(sc, e.getEntityThrowable());
            });
        }           
    }
    
    public static void onEntityItemProjectileHit(EntityItemProjectile ent, EntityPlayer p, ItemStack stack, List<Entity> ents)
    {        
        handleEvent(p, "item_hit", (sc) -> 
        {
            ScriptVars.setEntityVars(sc, ent);
            ScriptVars.setItemVars(sc, stack);
            sc.setVar("entities", ents);
        });      
    }
    
    @SubscribeEvent
    public void onEntityShear(PlayerInteractEvent.EntityInteract e)
    {              
        if(e.getHand() == EnumHand.OFF_HAND || !(e.getTarget() instanceof EntitySheep))
        {
            return;
        }
        EntityPlayer p = e.getEntityPlayer();   
        if(p.getHeldItemMainhand().getItem() != Items.SHEARS)
        {
            return;
        }
        EntitySheep sheep = (EntitySheep) e.getTarget();
        handleEvent(p, "entity_shear", (sc) -> 
        {
            ScriptVars.setEntityVars(sc, sheep);
            sc.setVar("entity_sheared", sheep.getSheared());
            sc.setVar("entity_color", sheep.getFleeceColor().toString());
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "entity_shear");
        });
    }
       
    @SubscribeEvent
    public void onBlockBreak(BlockEvent.BreakEvent e)
    {        
        handleEvent(e.getPlayer(), "block_break", (sc) -> 
        {
            ScriptVars.setBlockVars(sc, e.getWorld(), e.getPos());
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "block_break");
        });
    }
    
    /*@SubscribeEvent
    public void onPlayerLogin(FML e)
    {    
        EntityPlayer p = e.player;
        if(p == null)
        {
            return;
        }
        handleEvent(p, "player_login", (sc) -> 
        {       
            sc.setVar("first_join", Utils.hasPlayedBefore(p));
            PlayerList list = KajetansMod.server.getPlayerList();
            sc.setVar("is_banned", list.getBannedPlayers().isBanned(p.getGameProfile()));
            sc.setVar("is_whitelisted", list.getWhitelistedPlayers().isWhitelisted(p.getGameProfile()));
        });
    }*/
    
    @SubscribeEvent
    public void onPlayerLogin(PlayerEvent.PlayerLoggedInEvent e)
    {    
        EntityPlayer p = e.player;
        if(p == null)
        {
            return;
        }
        handleEvent(p, "player_login", (sc) -> 
        {       
            sc.setVar("first_join", Utils.hasPlayedBefore(p));
            PlayerList list = KajetansMod.server.getPlayerList();
            sc.setVar("is_banned", list.getBannedPlayers().isBanned(p.getGameProfile()));
            sc.setVar("is_whitelisted", list.getWhitelistedPlayers().isWhitelisted(p.getGameProfile()));
        });
    }
    
    @SubscribeEvent
    public void onPlayerLogout(PlayerEvent.PlayerLoggedOutEvent e)
    {    
        EntityPlayer p = e.player;
        if(p == null || e.player.ticksExisted < 20)
        {
            return;
        }
        handleEvent(p, "player_logout", sc -> {});
    }
    
    @SubscribeEvent
    public void onBucketFill(FillBucketEvent e)
    {      
        handleEvent(e.getEntityPlayer(), "bucket_use", (sc) -> 
        {
            RayTraceResult ray = e.getTarget();
            if(ray != null)
            {
                BlockPos pos = ray.getBlockPos();
                if(pos != null)
                {
                    sc.setVar("has_block", true); 
                    ScriptVars.setBlockVars(sc, e.getWorld(), pos);
                }
                else
                {
                    sc.setVar("has_block", false); 
                }
            }
            else
            {
                sc.setVar("has_block", false); 
            }
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "bucket_use");
        });
    } 
       
    @SubscribeEvent
    public void onBlockPlace(BlockEvent.PlaceEvent e)
    {
        handleEvent(e.getPlayer(), "block_place", (sc) -> 
        {
            sc.setVar("block_type_after", e.getPlacedBlock().getBlock().getRegistryName());
            ScriptVars.setBlockVars(sc, e.getWorld(), e.getPos());
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "block_place");
        });
    }    
    
    @SubscribeEvent
    public void onRightClickBlock(PlayerInteractEvent.RightClickBlock e)
    {
        if(e.getHand() == EnumHand.OFF_HAND)
        {
            return;
        }
        handleEvent(e.getEntityPlayer(), "block_click", (sc) -> 
        {
            sc.setVar("action", "right");
            ScriptVars.setBlockVars(sc, e.getWorld(), e.getPos());
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "block_click");
        });       
    }
    
    @SubscribeEvent
    public void onLeftClickBlock(PlayerInteractEvent.LeftClickBlock e)
    {
        if(e.getHand() == EnumHand.OFF_HAND)
        {
            return;
        }
        handleEvent(e.getEntityPlayer(), "block_click", (sc) -> 
        {
            sc.setVar("action", "left");
            ScriptVars.setBlockVars(sc, e.getWorld(), e.getPos());
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "block_click");
        });       
    }
    
    @SubscribeEvent(receiveCanceled = true)
    public void onEntityClick(PlayerInteractEvent.EntityInteract e)
    {
        if(e.getHand() != EnumHand.OFF_HAND)
        {
            handleEvent(e.getEntityPlayer(), "entity_click", (sc) -> 
            {
                ScriptVars.setEntityVars(sc, e.getTarget());
                sc.setVar("cancel", e.isCanceled()); 
            }, (sc) -> 
            {
                simpleCancel(sc, e, "entity_click");
            });
        }
    }
    
    @SubscribeEvent
    public void onFishing(ItemFishedEvent e)
    {
        handleEvent(e.getEntityPlayer(), "fishing", (sc) -> 
        {
            sc.setVar("drops", e.getDrops());
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "fishing");
        });
    }
    
    @SubscribeEvent
    public void onItemClick(PlayerInteractEvent.RightClickItem e)
    {        
        handleEvent(e.getEntityPlayer(), "item_air_click", (sc) -> 
        {
            ScriptVars.setItemVars(sc, e.getItemStack());
            sc.setVar("hand", e.getHand().toString());
            sc.setVar("cancel", e.isCanceled());  
        }, (sc) -> 
        {
            simpleCancel(sc, e, "item_air_click");
        });
    }
    
    public void onArmSwing(EntityPlayer p, EnumHand hand)
    {        
        handleEvent(p, "arm_swing", (sc) -> 
        {
            sc.setVar("hand", hand);
        });
    }
    
    @SubscribeEvent
    public void onItemUseStart(LivingEntityUseItemEvent.Start e)
    {        
        if(!(e.getEntityLiving() instanceof EntityPlayer))
        {
            return;
        }
        handleEvent((EntityPlayer) e.getEntityLiving(), "item_use_start", (sc) -> 
        {
            sc.setVar("duration", (double) e.getDuration());
            sc.setVar("cancel", e.isCanceled());  
        }, (sc) -> 
        {
            try
            {
                e.setDuration(sc.getVar("duration").getInt(sc));
                e.setCanceled(sc.getVar("cancel").getBoolean(sc)); 
            }
            catch(Exception ex)
            {
                KajetansMod.scripts.logger.print("invalid var in 'item_use_start' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
            }
        });
    }
    
    @SubscribeEvent
    public void onConsuming(LivingEntityUseItemEvent.Finish e)
    {
        if(!(e.getEntityLiving() instanceof EntityPlayer))
        {
            return;
        }
        handleEvent((EntityPlayer) e.getEntityLiving(), "item_use_finish", (sc) -> 
        {
            sc.setVar("result_stack", e.getResultStack());  
        });
    }
    
    @SubscribeEvent
    public void onCrafting(PlayerEvent.ItemCraftedEvent e)
    {  
        handleEvent(e.player, "craft", (sc) -> 
        {
            ScriptVars.setItemVars(sc, e.crafting);
        });   
    } 
    
    @SubscribeEvent
    public void onItemDrop(ItemTossEvent e)
    {
        handleEvent(e.getPlayer(), "player_toss", (sc) -> 
        {
            ScriptVars.setItemVars(sc, e.getEntityItem().getItem());   
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "player_toss");
        });
    }
    
    @SubscribeEvent
    public void onItemPickup(EntityItemPickupEvent e)
    {
        handleEvent(e.getEntityPlayer(), "player_pickup", (sc) -> 
        {
            ScriptVars.setEntityVars(sc, e.getItem()); 
            ScriptVars.setItemVars(sc, e.getItem().getItem());  
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "player_pickup");
        });
    }
    
    @SubscribeEvent
    public void onVehicleEnter(EntityMountEvent e)
    {
        if(!(e.getEntityMounting() instanceof EntityPlayer))
        {
            return;
        }
        EntityPlayer p = (EntityPlayer) e.getEntityMounting();
        handleEvent(p, "entity_mount", (sc) -> 
        {
            sc.setVar("mounting", e.isMounting()); 
            ScriptVars.setEntityVars(sc, e.getEntityBeingMounted()); 
            sc.setVar("cancel", e.isCanceled()); 
        }, (sc) -> 
        {
            simpleCancel(sc, e, "entity_mount");
        });
    }
    
    @SubscribeEvent
    public void onPlayerUsePortal(PlayerEvent.PlayerChangedDimensionEvent e)
    {
        handleEvent(e.player, "portal", (sc) -> 
        {
            sc.setVar("from", ModDimensions.getWorldName(e.fromDim));
            sc.setVar("to", ModDimensions.getWorldName(e.toDim));
        });
    }
    
    @SubscribeEvent
    public void onCommand(CommandEvent e)
    {
        if(e.getSender() instanceof EntityPlayer)
        {
            handleEvent((EntityPlayer) e.getSender(), "command", (sc) -> 
            {
                sc.setVar("command", e.getCommand().getName()); 
                sc.setVar("args", Arrays.stream(e.getParameters()).map(s -> Compiler.convert(s)).collect(Collectors.toList()));
                sc.setVar("cancel", e.isCanceled()); 
            }, (sc) -> 
            {
                simpleCancel(sc, e, "command");
            });
        }
    }
    
    public static void onCustomCommand(EntityPlayer p, String command, String[] args)
    {
        handleEvent(p, "custom_command", (sc) -> 
        {
            sc.setVar("command", command);
            if(args.length == 0)
            {
                sc.setVar("args", new ArrayList<>());
            }
            else
            {
                sc.setVar("args", Arrays.stream(args).map(s -> Compiler.convert(s)).collect(Collectors.toList()));
            }
        });
    }
    
    public static void onFunctionKey(EntityPlayerMP p, int key)
    {        
        handleEvent(p, "function_key", (sc) -> 
        {
            sc.setVar("key", (double) key);
        });      
    }
    
    @SubscribeEvent
    public void onChatEvent(ServerChatEvent e)
    {    
        handleEvent(e.getPlayer(), "chat", (sc) -> 
        {
            sc.setVar("message", Utils.colorMessage(e.getMessage(), e.getPlayer()));
            sc.setVar("cancel", e.isCanceled());
        }, (sc) -> 
        {
            try
            {
                e.setComponent(new TextComponentString(sc.getVar("message").getString(sc)));
                e.setCanceled(sc.getVar("cancel").getBoolean(sc)); 
            }
            catch(Exception ex)
            {
                KajetansMod.scripts.logger.print("invalid var in 'chat' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
            }
        });
    }
    
    @SubscribeEvent
    public void onGetPlayerName(net.minecraftforge.event.entity.player.PlayerEvent.NameFormat e)
    {
        handleEvent(e.getEntityPlayer(), "display_name", (sc) -> 
        {
            sc.setVar("display", e.getDisplayname());
        }, (sc) -> 
        {
            try
            {
                e.setDisplayname(sc.getVar("display").getString(sc));
            }
            catch(Exception ex)
            {
                KajetansMod.scripts.logger.print("invalid var in 'display_name' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
            }
        });
    }
    
    @SubscribeEvent
    public void onGetPlayerName(PlayerTabListNameEvent e)
    {
        handleEvent(e.getEntityPlayer(), "tab_list", (sc) -> 
        {
            sc.setVar("tab_name", e.getEntityPlayer().getName());
        }, (sc) -> 
        {
            try
            {
                e.setName(new TextComponentString("tab_name"));
            }
            catch(Exception ex)
            {
                KajetansMod.scripts.logger.print("invalid var in 'tab_list' event", ex, null, sc.getName(), sc, sc.getActiveRealLine());
            }
        });
    }
    
    @SubscribeEvent(priority = EventPriority.HIGHEST)
    public void BlockExplosionProtection(ExplosionEvent.Start e)
    {
        e.setCanceled(true);
        
        handleEvent(null, "explosion", (sc) -> 
        {
            sc.setVar("loc", new Location(e.getWorld(), e.getExplosion().getPosition()));
            sc.setVar("cancel", e.isCanceled());
        }, (sc) -> 
        {
            simpleCancel(sc, e, "explosion");
        });
    }
}
